<?php
/* 
Description: Core Library Admin Page functions
 
Copyright 2015 Malcolm Shergold

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

if (!class_exists('StageShowLibCalendarClass')) 
{
	class StageShowLibCalendarClass 
	{
		var $cssRoot = 'calendar';
		var $secsPerDay;
		var $linksOpenNewTab = true;
		var $sideCellIDs = array();
		
		var $dateNow;		// Timestamp of noon today
		var $startDate;		// Timestamp of noon on the first day shown on the calendar
		var $monthsInRow = 1;
		
		var $activeMonths = false;
		
		var $myDBaseObj;
		
		function __construct($myDBaseObj)
		{		
			$this->myDBaseObj = $myDBaseObj;
			
			$this->secsPerDay = (60*60*24);
				 
			$calStartOfWeek = $myDBaseObj->getOption('CalendarStartDay');
			if (!is_numeric($calStartOfWeek))
			{
		  		$calStartOfWeek = get_option( 'start_of_week' );
			}
  		
			// Get current day of the week
			$this->startDate = $this->dateNow = strtotime(date(StageShowLibDBaseClass::MYSQL_DATE_FORMAT).' 12:00:00');

			// Find date of Start of the Week
			$weekday = date( "N", $this->startDate);
			while ($weekday != $calStartOfWeek)
			{
				$this->startDate -= $this->secsPerDay;
				$weekday = date( "w", $this->startDate);
			}
		}

		function GetDateValue($dateNow)
		{
			// Get date in same format as returned by GetRecordDate()
			// Note: Default is MYSQL formatted date string
			return date(StageShowLibDBaseClass::MYSQL_DATE_FORMAT, $dateNow);			
		}
		
		function OutputCalender($results, $months=-1)
		{		
			$lastDay = 0;	
			$monthActive = false;						 
			$htmlOutput = '';
				
			// Get default class	 
			$dateCellClass = $this->cssRoot.'Date ';		
			$blockClass = $this->cssRoot.'Block ';		
			
			$cellRepeats = 1;
			$cellClass = '';
				
			$eventIndex = 0;
			$eventCount = count($results);
			
			// Get the events list (filtered) starting with last monday
			$calEntryDate = $this->startDate;
			
			// Find the first record with a date within the range of the calendar
			for ($eventIndex=0; $eventIndex<$eventCount; $eventIndex++)
			{
				if ($eventIndex===0)
				{
					$result = current($results);
				}
				else
				{
					$result = next($results);
				}
				$recordTimestamp = $this->GetRecordTimestamp($result);
				if ($recordTimestamp >= $calEntryDate) break;
			} 

			$newMonth = true;
			
			$htmlMonth = '';
			$htmlMonths = array();
			
			// Loop Round for 52 weeks (max)
			for ($weekNo = 1; $weekNo<=52; $weekNo++)
			{
				if ($newMonth)
				{
					//if ($result === null) break;
					if ($months-- == 0)
						break;
						
					if ($weekNo != 1) 
					{
						// Add end of table tag block
						$htmlMonth .=  "</tbody></table></div></div>\n";
						if (!$this->activeMonths || $monthActive)
							$htmlMonths[] = $htmlMonth;
						$monthActive = false;
					}
					
					$htmlMonth =  $this->OutputHeader($calEntryDate);
					$newMonth = false;
				}
					
				$htmlMonth .=  "<tr class=rowstart>\n";			
				
				$htmlWeek = '';
				
				// Loop Round for 7 days
				for ($dayNo = 1; $dayNo<=7; $dayNo++)
				{				
					if ($dayNo == 1)
					{
						foreach ($this->sideCellIDs as $sideCellIndex => $sideCellID)
						{
							$sideCell = $this->GetSideCellValue($result, $sideCellIndex);
							$sideCellClass = $this->GetSideCellClass($sideCellIndex, $result);
							$htmlMonth .=  '<td class="'.$sideCellClass.'">'.$sideCell.'</td>';							
						}
					}
					
					$dayOfMonth = date( "j", $calEntryDate);
					
					// Set default cell text
					$cellLink = $dayOfMonth;

					$dateNowVal = $this->GetDateValue($calEntryDate);
				
					// Loop Through all events with the same date
					$cellAltTag = '';
					while ($eventIndex < $eventCount)
					{
						if ($this->GetRecordDate($result) != $dateNowVal)
							break; 
						
						$cellClass = $this->DateTileClass($result);
						
						if ($cellRepeats <= 1)
						{
							if ($cellAltTag != '') $cellAltTag .= "\n";
							$cellAltTag .= $this->DateTileTitle($result);
							
							$cellURL = $this->DateTileURL($result);
							if ($cellURL !== '') 
							{
								$monthActive = true;
								$cellLink = "<strong>$dayOfMonth</strong>";
									
								$cellTarget = ($this->linksOpenNewTab) ? ' target="_blank" ' : '';
								
								// TODO - This link will only go to the last entry in the database that matches
								$cellURL = ' href="'.$cellURL.'" ';
								$cellLink =  '<a '.$cellURL.$cellTarget.' alt="'.$cellAltTag.'"  title="'.$cellAltTag.'">'.$cellLink.'</a>';							
							}
							
							$cellRepeats = $this->GetRecordDuration($result);
						}
						
						$eventIndex++;
						if ($eventIndex < $eventCount)
						{
							$result = next($results);
							$nextCellRepeats = $this->GetRecordDuration($result);
							if ($cellRepeats < $nextCellRepeats)
							{
								$cellRepeats = $nextCellRepeats;
							}
						}
					}						
					
					$cellRepeats--;
					$htmlWeek .=  '<td class="'.$dateCellClass.$cellClass.'">'.$cellLink.'</td>'."\n";

					if ($lastDay > $dayOfMonth)
						$newMonth = true;
									
					$lastDay = $dayOfMonth;
							 
					$calEntryDate += $this->secsPerDay;
					
					if (date( "j", $calEntryDate) == 1)
					{
						$newMonth = true;
						$lastDay = 1;
					}
				}
				$htmlMonth .=  $htmlWeek."</tr>\n";
			}			
			$htmlMonth .=  "</tbody></table></div></div>\n";	
			
			if (!$this->activeMonths || $monthActive)
				$htmlMonths[] = $htmlMonth;

			$htmlOutput = '';
			if ($this->monthsInRow > 1) $htmlOutput .= "<table class=coltable>\n";
			for ($month=0, $colNo=0; $month<count($htmlMonths); )
			{
				$htmlOutput .= "<tr class=debugrow>\n";
				for ($colNo=0; $colNo<$this->monthsInRow; $colNo++)
				{
					$htmlOutput .= '<td class="'.$blockClass.'">'."\n";
					$htmlOutput .= isset($htmlMonths[$month]) ? $htmlMonths[$month] : '&nbsp;';				
					$htmlOutput .= "</td>\n";
					$month++;
				}
				$htmlOutput .= "</tr>\n";
			}		
			if ($this->monthsInRow > 1) $htmlOutput .= "</table>\n";
			
			return $htmlOutput;
		}
		
		function OutputHeader( $headerDate )
		{	
			static $blockCount = 1;
			
			$htmlOutput  =  '';

			$blockClass = $this->cssRoot.'MonthBlock '.$this->cssRoot.'MonthBlock'.$blockCount++;
			
			$htmlOutput .= '<div class="'.$blockClass.'">'."\n";						
			$htmlOutput .= '<div class="'.$this->cssRoot.'Month">'.date( "F ", $headerDate)."</div>\n";						
			$htmlOutput .= '<div class="'.$this->cssRoot.'">';						
			$htmlOutput .= '<table class="'.$this->cssRoot.'Table"><tbody>'."\n<tr>\n";
			//$htmlOutput = '<table cellspacing="0" cellpadding="2" bordercolor="#CCCCCC" border="1" align="center" width="399"><tbody>';			
			
			foreach ($this->sideCellIDs as $sideCellIndex => $sideCellID)
			{
				$sideCellClass = $this->GetSideCellClass($sideCellIndex);
				$htmlOutput .=  '<td class="'.$sideCellClass.'">'.$sideCellID.'</td>';							
			}
						
			// Loop Round for 7 days
			for ($dayNo = 1; $dayNo<=7; $dayNo++)
			{
				$cellLink = date( "D", $headerDate);
							 
				$htmlOutput .=  '<td class="'.$this->cssRoot.'Day">'.$cellLink.'</td>';
				$htmlOutput .=  "\n";

				$headerDate += $this->secsPerDay;
			}
			
			$htmlOutput .=  "</tr>";
			return $htmlOutput;
		}
		
		function GetRecordTimestamp($result)
		{
			// This function must be defined in a derived class
			return 0;
		}
		
		function GetRecordDate($result)
		{
			$timestamp = $this->GetRecordTimestamp($result);
			
			// Strip Time from Date & Time
			$recordDate = date(StageShowLibDBaseClass::MYSQL_DATE_FORMAT, $timestamp);
			return $recordDate;
		}
					
		function GetRecordDuration($result)
		{
			return 1;
		}
					
		function DateTileClass($result)
		{
			return '';
		}
		
		function DateTileTitle($result)
		{
			return '';
		}
		
		function DateTileURL($result)
		{
			return '';
		}
		
		function GetSideCellValue($result, $sideCellIndex)
		{
			return '';
		}
					
		function GetSideCellClass($sideCellIndex, $result = array())
		{
			return '';
		}
		
	}
} //End Class StageShowLibCalendarClass

?>